ChunkDataRef

Struct ChunkDataRef 

pub struct ChunkDataRef<'a> {
    pub data: CompressedChunk<'a>,
    pub compression: Compression,
}
A reference to a piece of chunk data.

Containing the compressed chunk data (wrapped in a type to really notify you that its not uncompressed yet).
And which compression format is used.

Use RegionReader::decompress_data_ref to manually decompress the data. You should prefer just RegionReader::chunk which decompresses the chunk automatically.

But if you for some reason get chunk data externally or from RegionReader::chunk_data, then this will be used.

Fields

data: CompressedChunk<'a>

The chunks nbt data that is compressed

can use CompressedChunk::as_ref to get the inner data.

compression: Compression

The compression format that was used to compress the chunks data