Skip to main contentModule write
Source - PackedChunk
- A chunk whose data is compressed and contains data about how the chunk should be written
- PendingCompressed
- A chunks data that is compressed, and what compression was used.
- PendingUncompressed
- A chunks data that is uncompressed, along side what compression to use
- RegionWriter
- Used to write raw chunk data to and then format it into the region file format (.mca)
- WritableChunk
- A chunk that is in the state between being written and modified.
- PendingData
- A chunks that is either compressed or uncompressed
- current_timestamp
- Gets the current time in unix epoch